@@ -126,10 +126,14 @@ lsodar <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
     if (! is.null(forcings))
       flist <- checkforcings(forcings,times,dllname,initforc,verbose,fcontrol)
 
-    rho <- NULL
     if (is.null(ipar)) ipar<-0
     if (is.null(rpar)) rpar<-0
     Eventfunc <- events$func
+    if (is.function(Eventfunc))
+      rho <- environment(Eventfunc)
+    else
+      rho <- NULL
+    
   } else {
     if(is.null(initfunc))
       initpar <- NULL # parameter initialisation not needed if function is not a DLL
@@ -203,8 +207,8 @@ lsodar <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
       if (!is.matrix(tmp))
       stop("Jacobian function, 'jacfunc' must return a matrix\n")
       dd <- dim(tmp)
-      if((jt ==4 && dd != c(bandup+banddown+1,n)) ||
-         (jt ==1 && dd != c(n,n)))
+      if((jt ==4 && any(dd != c(bandup+banddown+1,n))) ||
+         (jt ==1 && any(dd != c(n,n))))
            stop("Jacobian dimension not ok")
     }
   }
@@ -259,7 +263,7 @@ lsodar <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
                as.integer(iwork), as.integer(jt),as.integer(Nglobal),
                as.integer(lrw),as.integer(liw),as.integer(IN),RootFunc,
                as.integer(nroot), as.double (rpar), as.integer(ipar),
-               as.integer(0), flist, events, lags, PACKAGE="deSolve")
+               0L, flist, events, lags, PACKAGE="deSolve")
 
 ### saving results
   iroot  <- attr(out, "iroot")
