@@ -10,17 +10,22 @@
 #include <R_ext/Applic.h>
 #include <R_ext/Rdynload.h>
 #include "deSolve.h"
-
-/*==================================================
-some functions for keeping track of how many SEXPs
-are PROTECTed, and UNPROTECTing them in the
-case of a FORTRAN stop.
-==================================================*/
- 
-long int N_Protected = 0; /* initialize this with zero at the first time */
+#include "externalptr.h"
 
 int solver_locked = 0;   /* prevent nested calls of odepack solvers */
 
+/* PROTECT / UNPROTECT now  strictly local, the following is no longer needed */
+
+/*==================================================
+ some functions for keeping track of how many SEXPs
+ are PROTECTed, and UNPROTECTing them in the
+ case of a FORTRAN stop.
+ ==================================================*/
+
+/*
+long int N_Protected = 0; // initialize this with zero at the first time
+
+
 void init_N_Protect(void) { N_Protected = 0; }
 
 void incr_N_Protect(void) { N_Protected++; }
@@ -30,23 +35,24 @@ void unprotect_all(void) { UNPROTECT((int) N_Protected); }
 long int save_N_Protected(void) {
   int saved_N = N_Protected;
   init_N_Protect();
-  return saved_N; 
+  return saved_N;
 }
 
 void restore_N_Protected(long int n) {
   unprotect_all();
-  N_Protected = n; 
+  N_Protected = n;
 }
 
 void my_unprotect(int n) {
     UNPROTECT(n);
     N_Protected -= n;
 }
+*/
 
 void lock_solver(void) {
   if (solver_locked) {
     /* important: unlock for the next call *after* error */
-    solver_locked = 0; 
+    solver_locked = 0;
     error("The used combination of solvers cannot be nested.\n");
   }
   solver_locked = 1;
@@ -81,35 +87,42 @@ SEXP de_gparms;
 SEXP initialisation functions
 =======================================================*/
 
+// obsolete
+/*
 void initglobals(int nt, int ntot) {
-/*  PROTECT(Time = NEW_NUMERIC(1));                  incr_N_Protect(); */
-  PROTECT(Y = allocVector(REALSXP,(n_eq)));        incr_N_Protect();
-  PROTECT(YOUT = allocMatrix(REALSXP,ntot+1,nt));  incr_N_Protect();
+  //  PROTECT(Time = NEW_NUMERIC(1)); incr_N_Protect();
+  PROTECT(Y = allocVector(REALSXP,(n_eq)));        //incr_N_Protect();
+  PROTECT(YOUT = allocMatrix(REALSXP,ntot+1,nt));  //incr_N_Protect();
 }
-
+*/
+// thpe: obsolete, 2017-07-17
+/*
 void initdaeglobals(int nt, int ntot) {
-/*  PROTECT(Time = NEW_NUMERIC(1));                    incr_N_Protect(); */
-  PROTECT(Rin  = NEW_NUMERIC(2));                    incr_N_Protect();
-  PROTECT(Y = allocVector(REALSXP,n_eq));            incr_N_Protect();
-  PROTECT(YPRIME = allocVector(REALSXP,n_eq));       incr_N_Protect();
-  PROTECT(YOUT = allocMatrix(REALSXP,ntot+1,nt));    incr_N_Protect();
+//  PROTECT(Time = NEW_NUMERIC(1));                    incr_N_Protect();
+  PROTECT(Rin  = NEW_NUMERIC(2));                    //incr_N_Protect();
+  PROTECT(Y = allocVector(REALSXP,n_eq));            //incr_N_Protect();
+  PROTECT(YPRIME = allocVector(REALSXP,n_eq));       //incr_N_Protect();
+  PROTECT(YOUT = allocMatrix(REALSXP,ntot+1,nt));    //incr_N_Protect();
 }
+*/
 
 /*======================================================
 Parameter initialisation functions
 note: forcing initialisation function is in forcings.c
 =======================================================*/
 
+// thpe: obsolete ??
+/*
 void initParms(SEXP Initfunc, SEXP Parms) {
   if (Initfunc == NA_STRING) return;
   if (inherits(Initfunc, "NativeSymbol")) {
     init_func_type *initializer;
-    PROTECT(de_gparms = Parms); incr_N_Protect();
-    initializer = (init_func_type *) R_ExternalPtrAddr(Initfunc);
+    PROTECT(de_gparms = Parms); //incr_N_Protect();
+    initializer = (init_func_type *) R_ExternalPtrAddrFn_(Initfunc);
     initializer(Initdeparms);
   }
 }
-
+*/
 
 void Initdeparms(int *N, double *parms) {
   int i, Nparms;
@@ -123,53 +136,55 @@ void Initdeparms(int *N, double *parms) {
     for (i = 0; i < *N; i++) parms[i] = REAL(de_gparms)[i];
   }
 }
-  
+
 SEXP get_deSolve_gparms(void) {
   return de_gparms;
 }
 
-/*=========================================================================== 
+/*===========================================================================
   C-equivalent of R-function timestep: gets the past and new time step
   =========================================================================== */
 SEXP getTimestep() {
   SEXP value;
   PROTECT(value = NEW_NUMERIC(2));
   if (timesteps == NULL) {         /* integration not yet started... */
-    for (int i = 0; i < 2; i++) 
+    for (int i = 0; i < 2; i++)
       NUMERIC_POINTER(value)[i] = 0.0;
   } else
-    for (int i = 0; i < 2; i++) 
+    for (int i = 0; i < 2; i++)
       NUMERIC_POINTER(value)[i] = timesteps[i];
   UNPROTECT(1);
   return(value);
 }
-  
-/*============================ ======================
- Termination 
+
+/*==================================================
+ Termination
 ===================================================*/
 
 /* an error occurred - save output in YOUT2 */
 void returnearly (int Print, int it, int ntot) {
   int j, k;
-  if (Print) 
+  if (Print)
     warning("Returning early. Results are accurate, as far as they go\n");
-  PROTECT(YOUT2 = allocMatrix(REALSXP,ntot+1,(it+2))); incr_N_Protect();
+  // thpe: protect before the call
+  //PROTECT(YOUT2 = allocMatrix(REALSXP,ntot+1,(it+2))); //incr_N_Protect();
   for (k = 0; k < it+2; k++)
     for (j = 0; j < ntot+1; j++)
       REAL(YOUT2)[k*(ntot+1) + j] = REAL(YOUT)[k*(ntot+1) + j];
-}   
+  //UNPROTECT(1); // thpe
+}
 
 /* add ISTATE and RSTATE */
-void terminate(int istate, int * iwork, int ilen, int ioffset, 
+void terminate(int istate, int * iwork, int ilen, int ioffset,
   double * rwork, int rlen, int roffset) {
 
   int k;
-  
-  PROTECT(ISTATE = allocVector(INTSXP, ilen)); incr_N_Protect();
+
+  //PROTECT(ISTATE = allocVector(INTSXP, ilen)); //incr_N_Protect();
   for (k = 0; k < ilen-1; k++) INTEGER(ISTATE)[k+1] = iwork[k +ioffset];
-  INTEGER(ISTATE)[0] = istate;  
-        
-  PROTECT(RWORK = allocVector(REALSXP, rlen)); incr_N_Protect();
+  INTEGER(ISTATE)[0] = istate;
+
+  //PROTECT(RWORK = allocVector(REALSXP, rlen)); //incr_N_Protect();
   for (k = 0; k < rlen; k++) REAL(RWORK)[k] = rwork[k+roffset];
   if (istate > 0) {
     setAttrib(YOUT, install("istate"), ISTATE);
@@ -182,6 +197,7 @@ void terminate(int istate, int * iwork, int ilen, int ioffset,
   /* timestep = 0 - for use in getTimestep */
   timesteps[0] = 0;
   timesteps[1] = 0;
+  //UNPROTECT(2); //thpe
 }
 
 /*==================================================
@@ -248,12 +264,12 @@ void initOutR(int isDll, int *nout, int *ntot, int neq, SEXP nOut, SEXP Rpar, SE
 
 void initOutC(int isDll, int *nout, int *ntot, int neq, SEXP nOut, SEXP Rpar, SEXP Ipar) {
   int j, lrpar, lipar;
-  /* initialise output when a dae ... */   
+  /* initialise output when a dae ... */
   /*  output always done here in C-code (<-> lsode, vode)... */
 
   *nout  = INTEGER(nOut)[0];
   *ntot  = n_eq+*nout;
-  
+
   if (isDll == 1) {                /* function is a dll */
     lrpar = *nout + LENGTH(Rpar);   /* length of rpar */
     lipar = 3    + LENGTH(Ipar);   /* length of ipar */
@@ -326,7 +342,7 @@ void sparsity2D (SEXP Type, int* iwork, int neq, int liw) {
       isp = i*Nt;
       for( j = 0; j < nx; j++) {
         for( k = 0; k < ny; k++) {
-          if (ij > liw-8-nspec)  
+          if (ij > liw-8-nspec)
             error("not enough memory allocated in iwork - increase liw %i ",liw);
           iwork[ij++] = m;
           if (k < ny-1)     iwork[ij++] = m+1;
@@ -355,7 +371,7 @@ void interact (int *ij, int nnz, int *iwork, int is, int ival) {
 
   isave = 1;
 /* check if not yet present for current state */
-	for (i = is; i < *ij; i++) 
+	for (i = is; i < *ij; i++)
 	  if (iwork[i] == ival) {
 	     isave = 0;
 	     break;
@@ -363,30 +379,30 @@ void interact (int *ij, int nnz, int *iwork, int is, int ival) {
 
 /* save */
 	if (isave == 1) {
-    if (*ij > nnz) 
+    if (*ij > nnz)
        error ("not enough memory allocated in iwork - increase liw %i ", nnz);
      iwork[(*ij)++] = ival;
   }
 }
 
 /*==================================================*/
-/* an element in C-array A(I,J,K), i=0,dim(1)-1 etc... is positioned at 
-   j*dim(2)*dim(3) + k*dim(3) + l + 1 in FORTRAN VECTOR! 
+/* an element in C-array A(I,J,K), i=0,dim(1)-1 etc... is positioned at
+   j*dim(2)*dim(3) + k*dim(3) + l + 1 in FORTRAN VECTOR!
    includes check on validity
 
-   dimens and boundary are reversed ... 
+   dimens and boundary are reversed ...
 */
 
 void sparsity3D (SEXP Type, int* iwork, int neq, int liw) {
     int nspec, nx, ny, nz, bndx, bndy, bndz, Nt, ij, is, isp, i, j, k, l, m, ll;
 
-    nspec = INTEGER(Type)[1]; 
-    nx    = INTEGER(Type)[2]; 
-    ny    = INTEGER(Type)[3]; 
-    nz    = INTEGER(Type)[4]; 
+    nspec = INTEGER(Type)[1];
+    nx    = INTEGER(Type)[2];
+    ny    = INTEGER(Type)[3];
+    nz    = INTEGER(Type)[4];
     bndx  = INTEGER(Type)[5];
-    bndy  = INTEGER(Type)[6]; 
-    bndz  = INTEGER(Type)[7]; 
+    bndy  = INTEGER(Type)[6];
+    bndz  = INTEGER(Type)[7];
 
     Nt    = nx*ny*nz;
     ij    = 31+neq;
@@ -398,38 +414,38 @@ void sparsity3D (SEXP Type, int* iwork, int neq, int liw) {
         for( k = 0; k < ny; k++) {
           for( ll = 0; ll < nz; ll++) {
             is = ij;
-            
-            if (ij > liw-6-nspec)  
+
+            if (ij > liw-6-nspec)
               error ("not enough memory allocated in iwork - increase liw %i ", liw);
                             interact (&ij, liw, iwork, is, m);
-            if (ll < nz-1)  
+            if (ll < nz-1)
               interact (&ij, liw, iwork, is, m+1);
             else if (bndz == 1)
-              interact (&ij, liw, iwork, is, isp + j*ny*nz + k*nz + 1);              
-            
-            if (k  < ny-1) 
+              interact (&ij, liw, iwork, is, isp + j*ny*nz + k*nz + 1);
+
+            if (k  < ny-1)
               interact (&ij, liw, iwork, is, m+nz);
             else  if (bndy == 1)
               interact (&ij, liw, iwork, is, isp + j*ny*nz + ll + 1);
-              
-            if (j  < nx-1)  
+
+            if (j  < nx-1)
               interact (&ij, liw, iwork, is, m+ny*nz);
-            else if (bndx == 1)  
+            else if (bndx == 1)
               interact (&ij, liw, iwork, is, isp + k*nz + ll + 1);
 
-            if (j > 0)      
+            if (j > 0)
               interact (&ij, liw, iwork, is, m-ny*nz);
             else if (bndx == 1)
               interact (&ij, liw, iwork, is, isp+(nx-1)*ny*nz+k*nz+ll+1);
-                                         
-            if (k > 0)  
+
+            if (k > 0)
               interact (&ij, liw, iwork, is, m-nz);
             else  if (bndy == 1)
-              interact (&ij, liw, iwork, is, isp + j*ny*nz+(ny-1)*nz+ll+1);              
-            
-            if (ll > 0) 
+              interact (&ij, liw, iwork, is, isp + j*ny*nz+(ny-1)*nz+ll+1);
+
+            if (ll > 0)
               interact (&ij, liw, iwork, is, m-1);
-            else if (bndz == 1)  
+            else if (bndz == 1)
               interact (&ij, liw, iwork, is, isp + j*ny*nz+k*nz+nz);
 
             for(l = 0; l < nspec; l++)
